@@ -1,4 +1,5 @@
 /**********************************************************************
+** Copyright (C) 2014 Riverbank Computing Limited
 ** Copyright (C) 2002-2007 Detlev Offenbach <detlev@die-offenbachs.de>
 **
 ** This is a modified version of lupdate. The original is part of Qt-Linguist.
@@ -13,22 +14,42 @@
 
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the Qt Linguist of the Qt Toolkit.
+** This file is part of the tools applications of the Qt Toolkit.
 **
-** Licensees holding a valid Qt License Agreement may use this file in
-** accordance with the rights, responsibilities and obligations
-** contained therein.  Please consult your licensing agreement or
-** contact sales@trolltech.com if any conditions of this licensing
-** agreement are not clear to you.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
-** Further information about Qt licensing is available at:
-** http://www.trolltech.com/products/qt/licensing.html or by
-** contacting info@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
+**
+**
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -64,6 +85,10 @@ enum { Tok_Eof, Tok_class, Tok_return, Tok_tr,
        Tok_LeftParen, Tok_RightParen,
        Tok_Comma, Tok_None, Tok_Integer};
 
+// The names of function aliases passed on the command line.
+static const char *tr_function;
+static const char *translate_function;
+
 /*
   The tokenizer maintains the following global variables. The names
   should be self-explanatory.
@@ -91,40 +116,74 @@ static FILE *yyInFile;
 // the string to read from and current position in the string (otherwise)
 static QString yyInStr;
 static int yyInPos;
-static int buf;
+// - 'rawbuf' is used to hold bytes before universal newline translation.
+// - 'buf' is its higher-level counterpart, where every end-of-line appears as
+//   a single '\n' character, regardless of the end-of-line style used in input
+//   files.
+static int buf, rawbuf;
 
 static int (*getChar)();
 static int (*peekChar)();
 
 static bool yyParsingUtf8;
 
+static int getTranslatedCharFromFile()
+{
+    int c;
+
+    if ( rawbuf < 0 )           // Empty raw buffer?
+        c = getc( yyInFile );
+    else {
+        c = rawbuf;
+        rawbuf = -1;            // Declare the raw buffer empty.
+    }
+
+    // Universal newline translation, similar to what Python does
+    if ( c == '\r' ) {
+        c = getc( yyInFile ); // Last byte of a \r\n sequence?
+        if ( c != '\n')
+            {
+                rawbuf = c; // No, put it in 'rawbuf' for later processing.
+                // Logical character that will be seen by higher-level functions
+                c = '\n';
+            }
+        // In all cases, c == '\n' here.
+    }
+
+    return c;
+}
+
 static int getCharFromFile()
 {
     int c;
 
-    if ( buf < 0 )
-        c = getc( yyInFile );
-    else {
+    if ( buf < 0 ) {            // Empty buffer?
+        c = getTranslatedCharFromFile();
+    } else {
         c = buf;
-        buf = -1;
+        buf = -1;               // Declare the buffer empty.
     }
-    if ( c == '\n' )
-        yyCurLineNo++;
+
+    if ( c == '\n' )            // This is after universal newline translation
+        yyCurLineNo++;          // (i.e., a "logical" newline character).
+
     return c;
 }
 
 static int peekCharFromFile()
 {
-    int c = getc( yyInFile );
-    buf = c;
-    return c;
+    // Read a character, possibly performing universal newline translation,
+    // and put it in 'buf' so that the next call to getCharFromFile() finds it
+    // already available.
+    buf = getCharFromFile();
+    return buf;
 }
 
 static void startTokenizer( const char *fileName, int (*getCharFunc)(),
                             int (*peekCharFunc)(), QTextCodec *codecForTr, QTextCodec *codecForSource )
 {
     yyInPos = 0;
-    buf = -1;
+    buf = rawbuf = -1;
     getChar = getCharFunc;
     peekChar = peekCharFunc;
 
@@ -165,6 +224,13 @@ static int getToken()
             } while ( isalnum(yyCh) || yyCh == '_' );
             yyIdent[yyIdentLen] = '\0';
 
+            // Check for names passed on the command line.
+            if (tr_function && strcmp(yyIdent, tr_function) == 0)
+                return Tok_tr;
+
+            if (translate_function && strcmp(yyIdent, translate_function) == 0)
+                return Tok_translate;
+
             bool might_be_str = false;
 
             switch ( yyIdent[0] ) {
@@ -312,18 +378,18 @@ static int getToken()
                             if ( yyCh == 'x' ) {
                                 QByteArray hex = "0";
 
-                            yyCh = getChar();
-                            while ( isxdigit(yyCh) ) {
-                                hex += (char) yyCh;
                                 yyCh = getChar();
-                            }
+                                while ( isxdigit(yyCh) ) {
+                                    hex += (char) yyCh;
+                                    yyCh = getChar();
+                                }
 #if defined(_MSC_VER) && _MSC_VER >= 1400
-                            sscanf_s( hex, "%x", &n );
+                                sscanf_s( hex, "%x", &n );
 #else
-                            sscanf( hex, "%x", &n );
+                                sscanf( hex, "%x", &n );
 #endif
-                            if ( yyStringLen < sizeof(yyString) - 1 )
-                                yyString[yyStringLen++] = (char) n;
+                                if ( yyStringLen < sizeof(yyString) - 1 )
+                                    yyString[yyStringLen++] = (char) n;
                             } else if ( yyCh >= '0' && yyCh < '8' ) {
                                 QByteArray oct = "";
                                 int n = 0;
@@ -340,6 +406,8 @@ static int getToken()
     #endif
                                 if ( yyStringLen < sizeof(yyString) - 1 )
                                     yyString[yyStringLen++] = (char) n;
+                            } else if ( yyCh == '\n' ) {
+                                yyCh = getChar();
                             } else {
                                 const char *p = strchr( tab, yyCh );
                                 if ( yyStringLen < sizeof(yyString) - 1 )
@@ -491,44 +559,6 @@ static bool matchString( QByteArray *s )
     return matches;
 }
 
-static bool matchEncoding( bool *utf8 )
-{
-    // Remove any leading module paths.
-    if (yyTok == Tok_Ident && strcmp(yyIdent, "PyQt4") == 0)
-    {
-    yyTok = getToken();
-
-    if (yyTok != Tok_Dot)
-        return false;
-
-    yyTok = getToken();
-    }
-
-    if (yyTok == Tok_Ident && (strcmp(yyIdent, "QtGui") == 0 || strcmp(yyIdent, "QtCore") == 0))
-    {
-    yyTok = getToken();
-
-    if (yyTok != Tok_Dot)
-        return false;
-
-    yyTok = getToken();
-    }
-
-    if ( yyTok == Tok_Ident ) {
-        if (strcmp(yyIdent, "QApplication") == 0 || strcmp(yyIdent, "QCoreApplication") == 0) {
-            yyTok = getToken();
-
-            if (yyTok == Tok_Dot)
-                yyTok = getToken();
-        }
-        *utf8 = QString( yyIdent ).endsWith( QString("UTF8") );
-        yyTok = getToken();
-        return true;
-    } else {
-        return false;
-    }
-}
-
 static bool matchStringOrNone(QByteArray *s)
 {
     bool matches = matchString(s);
@@ -608,6 +638,8 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                     com = "";
                     bool plural = false;
 
+                    // Note that this isn't as rigorous as the parsing of
+                    // translate() below.
                     if (match(Tok_RightParen))
                     {
                         // There is no comment or plural arguments.
@@ -648,7 +680,6 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                 }
                 break;
             case Tok_translate:
-                utf8 = false;
                 yyTok = getToken();
                 if ( match(Tok_LeftParen) &&
                  matchString(&context) &&
@@ -660,23 +691,12 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                         // look for comment
                         if ( match(Tok_Comma) && matchStringOrNone(&com)) {
                             if (!match(Tok_RightParen)) {
-                                // look for encoding
+                                // Look for n.
                                 if (match(Tok_Comma)) {
-                                    if (matchEncoding(&utf8)) {
-                                        if (!match(Tok_RightParen)) {
-                                            // look for the plural quantifier,
-                                            // this can be a number, an identifier or a function call,
-                                            // so for simplicity we mark it as plural if we know we have a comma instead of an
-                                            // right parentheses.
-                                            plural = match(Tok_Comma);
-                                        }
+                                    if (matchExpression() && match(Tok_RightParen)) {
+                                        plural = true;
                                     } else {
-                                        // This can be a QTranslator::translate("context", "source", "comment", n) plural translation
-                                        if (matchExpression() && match(Tok_RightParen)) {
-                                            plural = true;
-                                        } else {
-                                            break;
-                                        }
+                                        break;
                                     }
                                 } else {
                                     break;
@@ -690,7 +710,7 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                     {
                         tor->insert( MetaTranslatorMessage(context, text, com, 
                                                         yyFileName, yyParenLineNo,
-                                                        QStringList(), utf8,
+                                                        QStringList(), true,
                                                         MetaTranslatorMessage::Unfinished,
                                                         plural) );
                     }
@@ -733,9 +753,14 @@ static void parse( MetaTranslator *tor, const char *initialContext,
             (const char *) yyFileName );
 }
 
-void fetchtr_py( const char *fileName, MetaTranslator *tor,
-                  const char *defaultContext, bool mustExist, const QByteArray &codecForSource )
+void fetchtr_py(const char *fileName, MetaTranslator *tor,
+        const char *defaultContext, bool mustExist,
+        const QByteArray &codecForSource, const char *tr_func,
+        const char *translate_func)
 {
+    tr_function = tr_func;
+    translate_function = translate_func;
+
 #if defined(_MSC_VER) && _MSC_VER >= 1400
     if (fopen_s(&yyInFile, fileName, "r")) {
         if ( mustExist ) {
