@@ -1,6 +1,6 @@
 // This defines the interfaces to various odd and ends.
 //
-// Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt5.
 // 
@@ -24,10 +24,10 @@
 
 #include <Python.h>
 
-#include "qpycore_sip.h"
+#include "sipAPIQtCore.h"
 
 
-bool qpycore_is_pyqt4_class(const sipTypeDef *td);
+bool qpycore_is_pyqt_class(const sipTypeDef *td);
 
 
 #endif
