@@ -1,6 +1,6 @@
 // This contains the implementation of the pyqtBoundSignal type.
 //
-// Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt5.
 // 
@@ -24,13 +24,16 @@
 #include <QByteArray>
 #include <QMetaObject>
 
+#include "qpycore_api.h"
 #include "qpycore_chimera.h"
 #include "qpycore_misc.h"
+#include "qpycore_objectified_strings.h"
 #include "qpycore_pyqtboundsignal.h"
 #include "qpycore_pyqtpyobject.h"
 #include "qpycore_pyqtsignal.h"
 #include "qpycore_pyqtslotproxy.h"
-#include "qpycore_sip.h"
+
+#include "sipAPIQtCore.h"
 
 
 // Forward declarations.
@@ -65,14 +68,20 @@ static QByteArray slot_signature_from_metaobject(
         const QByteArray &slot_name, int nr_args);
 static QByteArray slot_signature(const Chimera::Signature *signal_signature,
         const QByteArray &slot_name, int nr_args);
+static sipErrorState get_receiver_slot_signature(PyObject *slot,
+        QObject *transmitter, const Chimera::Signature *signal_signature,
+        bool single_shot, QObject **receiver, QByteArray &slot_signature,
+        bool unique_connection_check, int no_receiver_check);
 
 
 // Doc-strings.
 PyDoc_STRVAR(pyqtBoundSignal_connect_doc,
-"connect(slot[, type=Qt.AutoConnection])\n"
+"connect(slot, type=Qt.AutoConnection, no_receiver_check=False)\n"
 "\n"
 "slot is either a Python callable or another signal.\n"
-"type is a Qt.ConnectionType");
+"type is a Qt.ConnectionType.\n"
+"no_receiver_check is True to disable the check that the receiver's C++\n"
+"instance still exists when the signal is emitted.\n");
 
 PyDoc_STRVAR(pyqtBoundSignal_disconnect_doc,
 "disconnect([slot])\n"
@@ -169,6 +178,9 @@ PyTypeObject qpycore_pyqtBoundSignal_Type = {
     0,                      /* tp_weaklist */
     0,                      /* tp_del */
     0,                      /* tp_version_tag */
+#if PY_VERSION_HEX >= 0x03040000
+    0,                      /* tp_finalize */
+#endif
 };
 
 
@@ -209,7 +221,7 @@ static PyObject *pyqtBoundSignal_get_signal(PyObject *self, void *)
 #else
         PyString_FromString
 #endif
-            (bs->unbound_signal->signature->signature.constData());
+            (bs->unbound_signal->parsed_signature->signature.constData());
 }
 
 
@@ -218,7 +230,7 @@ static PyObject *pyqtBoundSignal_repr(PyObject *self)
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
-    QByteArray name = bs->unbound_signal->signature->name();
+    QByteArray name = bs->unbound_signal->parsed_signature->name();
 
     return
 #if PY_MAJOR_VERSION >= 3
@@ -299,13 +311,20 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
     static const char *kwds[] = {
         "slot",
         "type",
+        "no_receiver_check",
         0
     };
 
     PyObject *py_slot, *py_type = 0;
+    int no_receiver_check = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwd_args, "O|O:connect",
-                const_cast<char **>(kwds), &py_slot, &py_type))
+    if (!PyArg_ParseTupleAndKeywords(args, kwd_args,
+#if PY_VERSION_HEX >= 0x03030000
+                "O|Op:connect",
+#else
+                "O|Oi:connect",
+#endif
+                const_cast<char **>(kwds), &py_slot, &py_type, &no_receiver_check))
         return 0;
 
     Qt::ConnectionType q_type = Qt::AutoConnection;
@@ -325,11 +344,13 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
     }
 
     QObject *q_tx = bs->bound_qobject, *q_rx;
-    Chimera::Signature *signal_signature = bs->unbound_signal->signature;
+    Chimera::Signature *signal_signature = bs->unbound_signal->parsed_signature;
     QByteArray slot_signature;
 
-    sipErrorState estate = qpycore_get_receiver_slot_signature(py_slot, q_tx,
-            signal_signature, false, &q_rx, slot_signature);
+    sipErrorState estate = get_receiver_slot_signature(py_slot, q_tx,
+            signal_signature, false, &q_rx, slot_signature,
+            ((q_type & Qt::UniqueConnection) == Qt::UniqueConnection),
+            no_receiver_check);
 
     if (estate != sipErrorNone)
     {
@@ -368,6 +389,18 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
 sipErrorState qpycore_get_receiver_slot_signature(PyObject *slot,
         QObject *transmitter, const Chimera::Signature *signal_signature,
         bool single_shot, QObject **receiver, QByteArray &slot_signature)
+{
+    return get_receiver_slot_signature(slot, transmitter, signal_signature,
+            single_shot, receiver, slot_signature, false, 0);
+}
+
+
+// Get the receiver object and slot signature from a callable or signal.
+// Optionally disable the receiver check.
+static sipErrorState get_receiver_slot_signature(PyObject *slot,
+        QObject *transmitter, const Chimera::Signature *signal_signature,
+        bool single_shot, QObject **receiver, QByteArray &slot_signature,
+        bool unique_connection_check, int no_receiver_check)
 {
     // See if the slot is a signal.
     if (PyObject_TypeCheck(slot, &qpycore_pyqtBoundSignal_Type))
@@ -375,7 +408,7 @@ sipErrorState qpycore_get_receiver_slot_signature(PyObject *slot,
         qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)slot;
 
         *receiver = bs->bound_qobject;
-        slot_signature = bs->unbound_signal->signature->signature;
+        slot_signature = bs->unbound_signal->parsed_signature->signature;
 
         return sipErrorNone;
     }
@@ -396,11 +429,29 @@ sipErrorState qpycore_get_receiver_slot_signature(PyObject *slot,
         // Create a proxy for the slot.
         PyQtSlotProxy *proxy;
 
+        if (unique_connection_check)
+        {
+            proxy = PyQtSlotProxy::findSlotProxy(transmitter,
+                    signal_signature->signature, slot);
+
+            if (proxy)
+            {
+                // We give more information than we could if it was a Qt slot
+                // but to be consistent we raise a TypeError even though it's
+                // not the most appropriate for the type of error.
+                PyErr_SetString(PyExc_TypeError, "connection is not unique");
+                return sipErrorFail;
+            }
+        }
+
         Py_BEGIN_ALLOW_THREADS
 
         proxy = new PyQtSlotProxy(slot, transmitter, signal_signature,
                 single_shot);
 
+        if (no_receiver_check)
+            proxy->disableReceiverCheck();
+
         if (proxy->metaObject())
         {
             if (*receiver)
@@ -457,7 +508,7 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
     PyObject *py_slot = 0, *res_obj;
-    Chimera::Signature *signal_signature = bs->unbound_signal->signature;
+    Chimera::Signature *signal_signature = bs->unbound_signal->parsed_signature;
 
     if (!PyArg_ParseTuple(args, "|O:disconnect", &py_slot))
         return 0;
@@ -479,7 +530,7 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
         qpycore_pyqtBoundSignal *slot_bs = (qpycore_pyqtBoundSignal *)py_slot;
 
         return disconnect(bs, slot_bs->bound_qobject,
-                slot_bs->unbound_signal->signature->signature.constData());
+                slot_bs->unbound_signal->parsed_signature->signature.constData());
     }
 
     if (!PyCallable_Check(py_slot))
@@ -523,7 +574,7 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
 static PyObject *disconnect(qpycore_pyqtBoundSignal *bs, QObject *qrx,
         const char *slot)
 {
-    Chimera::Signature *signature = bs->unbound_signal->signature;
+    Chimera::Signature *signature = bs->unbound_signal->parsed_signature;
     bool ok;
 
     Py_BEGIN_ALLOW_THREADS
@@ -565,32 +616,47 @@ static PyObject *pyqtBoundSignal_emit(PyObject *self, PyObject *args)
 
     if (!bs->bound_qobject->signalsBlocked())
     {
-        Chimera::Signature *signature = bs->unbound_signal->signature;
-        int mo_index = bs->bound_qobject->metaObject()->indexOfSignal(signature->signature.constData() + 1);
+        Q_ASSERT(PyTuple_Check(args));
 
-        if (mo_index < 0)
+        qpycore_pyqtSignal *ps = bs->unbound_signal;
+
+        // Use the emitter if there is one.
+        if (ps->emitter)
         {
-            PyErr_Format(PyExc_AttributeError,
-                    "signal was not defined in the first super-class of class '%s'",
-                    Py_TYPE(bs->bound_pyobject)->tp_name);
-            return 0;
-        }
-
-        // Use the docstring if there is one and it is auto-generated.
-        const char *docstring = bs->unbound_signal->docstring;
-
-        if (!docstring || *docstring != '\1')
-        {
-            docstring = signature->py_signature.constData();
+            if (ps->emitter(bs->bound_qobject, args) < 0)
+                return 0;
         }
         else
         {
-            // Skip the auto-generated marker.
-            ++docstring;
-        }
+            Chimera::Signature *signature = ps->parsed_signature;
+            int mo_index = bs->bound_qobject->metaObject()->indexOfSignal(
+                    signature->signature.constData() + 1);
 
-        if (!do_emit(bs->bound_qobject, mo_index, signature, docstring, args))
-            return 0;
+            if (mo_index < 0)
+            {
+                PyErr_Format(PyExc_AttributeError,
+                        "'%s' does not have a signal with the signature %s",
+                        Py_TYPE(bs->bound_pyobject)->tp_name,
+                        signature->signature.constData() + 1);
+                return 0;
+            }
+
+            // Use the docstring if there is one and it is auto-generated.
+            const char *docstring = bs->unbound_signal->docstring;
+
+            if (!docstring || *docstring != '\1')
+            {
+                docstring = signature->py_signature.constData();
+            }
+            else
+            {
+                // Skip the auto-generated marker.
+                ++docstring;
+            }
+
+            if (!do_emit(bs->bound_qobject, mo_index, signature, docstring, args))
+                return 0;
+        }
     }
 
     Py_INCREF(Py_None);
@@ -685,7 +751,7 @@ static bool get_receiver(PyObject *slot,
         Py_DECREF(f_name_obj);
 
         // See if this has been decorated.
-        decorations = PyObject_GetAttr(f, qpycore_signature_attr_name);
+        decorations = PyObject_GetAttr(f, qpycore_dunder_pyqtsignature);
 
         if (decorations)
         {
