@@ -1,7 +1,7 @@
 // This is the definition and implementation of the
 // QPyDesignerCustomWidgetCollectionPlugin class.
 //
-// Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt5.
 // 
@@ -24,7 +24,7 @@
 
 
 #include <QObject>
-#include <QDesignerCustomWidgetCollectionInterface>
+#include <QtDesigner>
 
 
 class QPyDesignerCustomWidgetCollectionPlugin : public QObject, public QDesignerCustomWidgetCollectionInterface
