@@ -1,6 +1,6 @@
 // This defines the API provided by this library.
 //
-// Copyright (c) 2013 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt5.
 // 
@@ -22,6 +22,10 @@
 #define _QPYQUICK_API_H
 
 
+// Python exception handling.
+typedef void (*pyqt5_qtquick_err_print_t)();
+extern pyqt5_qtquick_err_print_t pyqt5_qtquick_err_print;
+
 // Initialisation.
 void qpyquick_post_init();
 
